@@ -1,6 +1,6 @@
 # This script configures QScintilla for PyQt v3 and/or v4.
 #
-# Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
 # This file is part of QScintilla.
 # 
@@ -16,13 +16,8 @@
 # GPL Exception version 1.1, which can be found in the file
 # GPL_EXCEPTION.txt in this package.
 # 
-# Please review the following information to ensure GNU General
-# Public Licensing requirements will be met:
-# http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-# you are unsure which license is appropriate for your use, please
-# review the following information:
-# http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-# or contact the sales department at sales@riverbankcomputing.com.
+# If you are unsure which license is appropriate for your use, please
+# contact the sales department at sales@riverbankcomputing.com.
 # 
 # This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 # WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -64,11 +59,11 @@ else:
 
 
 # This must be kept in sync with qscintilla.pro.
-QSCI_API_MAJOR = 5
+QSCI_API_MAJOR = 8
 
 
 # Initialise the globals.
-sip_min_version = 0x040a00
+sip_min_version = 0x040c00
 
 if sys.platform == "win32":
     qsci_define = "QSCINTILLA_DLL"
@@ -89,7 +84,7 @@ def create_optparser():
         setattr(parser.values, option.dest, os.path.abspath(value))
 
     p = optparse.OptionParser(usage="python %prog [options]",
-            version="2.4.3")
+            version="2.6")
 
     p.add_option("-a", "--apidir", action="callback", default=None,
             type="string", metavar="DIR", dest="qscidir",
@@ -132,6 +127,25 @@ def create_optparser():
             metavar="DIR", dest="qscisipdir", callback=store_abspath,
             type="string", help="where the QScintilla .sip files will be "
             "installed [default: %s]" % pyqt.pyqt_sip_dir)
+    p.add_option("-T", "--no-timestamp", action="store_true", default=False,
+            dest="no_timestamp", help="suppress timestamps in the header "
+            "comments of generated code [default: include timestamps]")
+
+    if sys.platform != 'win32':
+        if sys.platform in ('linux2', 'darwin'):
+            pip_default = True
+            pip_default_str = "enabled"
+        else:
+            pip_default = False
+            pip_default_str = "disabled"
+
+        p.add_option("--protected-is-public", action="store_true",
+                default=pip_default, dest="prot_is_public",
+                help="enable building with 'protected' redefined as 'public' "
+                        "[default: %s]" % pip_default_str)
+        p.add_option("--protected-not-public", action="store_false",
+                dest="prot_is_public",
+                help="disable building with 'protected' redefined as 'public'")
 
     return p
 
@@ -152,6 +166,9 @@ def inform_user():
     else:
         sipconfig.inform("The QScintilla module is being built with generated docstrings.")
 
+    if opts.prot_is_public:
+        sipconfig.inform("The QScintilla module is being built with 'protected' redefined as 'public'.")
+
 
 def check_qscintilla():
     """See if QScintilla can be found and what its version is.
@@ -167,8 +184,8 @@ def check_qscintilla():
             # Because we include the Python bindings with the C++ code we can
             # reasonably force the same version to be used and not bother about
             # versioning.
-            if sciversstr != "2.4.3":
-                sipconfig.error("QScintilla %s is being used but the Python bindings 2.4.3 are being built.  Please use matching versions." % sciversstr)
+            if sciversstr != "2.6":
+                sipconfig.error("QScintilla %s is being used but the Python bindings 2.6 are being built.  Please use matching versions." % sciversstr)
 
             sipconfig.inform("QScintilla %s is being used." % sciversstr)
         else:
@@ -214,9 +231,15 @@ def generate_code():
 
     argv.extend(sip_flags())
 
+    if opts.no_timestamp:
+        argv.append("-T")
+
     if not opts.no_docstrings:
         argv.append("-o");
 
+    if opts.prot_is_public:
+        argv.append("-P");
+
     if opts.concat:
         argv.append("-j")
         argv.append(str(opts.split))
@@ -281,7 +304,10 @@ def generate_code():
             installs=installs,
             static=opts.static,
             debug=opts.debug,
-            universal=pyqt.universal
+            universal=pyqt.universal,
+            arch=pyqt.arch,
+            prot_is_public=opts.prot_is_public,
+            deployment_target=pyqt.deployment_target
         )
     else:
         makefile = Makefile(
@@ -325,6 +351,10 @@ def main(argv):
         p.print_help()
         sys.exit(2)
 
+    # Provide defaults for platform-specific options.
+    if sys.platform == 'win32':
+        opts.prot_is_public = False
+
     if opts.not_dll:
         global qsci_define
         qsci_define = ""
