@@ -1,6 +1,6 @@
 // This class defines the "official" low-level API.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -45,11 +40,14 @@ extern "C++" {
 #include <Qsci/qsciglobal.h>
 
 
+QT_BEGIN_NAMESPACE
 class QColor;
+class QImage;
 class QPainter;
 class QPixmap;
 class QObject;
 class QScrollBar;
+QT_END_NAMESPACE
 
 class ScintillaQt;
 
@@ -235,7 +233,8 @@ public:
         //! \a wParam is the number of the marker.
         //! \a lParam is the marker symbol and is one of the SC_MARK_* values.
         //!
-        //! \sa SCI_MARKERADD, SCI_MARKERDEFINEPIXMAP
+        //! \sa SCI_MARKERADD, SCI_MARKERDEFINEPIXMAP,
+        //! SCI_MARKERDEFINERGBAIMAGE
         SCI_MARKERDEFINE = 2040,
 
         //! This message sets the foreground colour used to draw a marker.  A
@@ -311,31 +310,31 @@ public:
         //! ports of Scintilla this is a pointer to either raw or textual XPM
         //! image data.
         //!
-        //! \sa SCI_MARKERDEFINE
+        //! \sa SCI_MARKERDEFINE, SCI_MARKERDEFINERGBAIMAGE
         SCI_MARKERDEFINEPIXMAP = 2049,
 
         //! This message sets what can be displayed in a margin.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //! \a lParam is the logical or of the SC_MARGIN_* values.
         //!
         //! \sa SCI_GETMARGINTYPEN
         SCI_SETMARGINTYPEN = 2240,
 
         //! This message returns what can be displayed in a margin.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //!
         //! \sa SCI_SETMARGINTYPEN
         SCI_GETMARGINTYPEN = 2241,
 
         //! This message sets the width of a margin in pixels.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //! \a lParam is the new margin width.
         //!
         //! \sa SCI_GETMARGINWIDTHN
         SCI_SETMARGINWIDTHN = 2242,
 
         //! This message returns the width of a margin in pixels.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //!
         //! \sa SCI_SETMARGINWIDTHN
         SCI_GETMARGINWIDTHN = 2243,
@@ -344,20 +343,20 @@ public:
         //! with one bit for each possible marker.  If a bit is set then the
         //! corresponding marker is displayed.  By default, all markers are
         //! displayed.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //! \a lParam is the new margin mask.
         //!
         //! \sa SCI_GETMARGINMASKN, SCI_MARKERDEFINE
         SCI_SETMARGINMASKN = 2244,
 
         //! This message returns the mask of a margin.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //!
         //! \sa SCI_SETMARGINMASKN
         SCI_GETMARGINMASKN = 2245,
 
         //! This message sets the sensitivity of a margin to mouse clicks.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //! \a lParam is non-zero to make the margin sensitive to mouse clicks.
         //! When the mouse is clicked the SCN_MARGINCLICK() signal is emitted.
         //!
@@ -365,11 +364,26 @@ public:
         SCI_SETMARGINSENSITIVEN = 2246,
 
         //! This message returns the sensitivity of a margin to mouse clicks.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //!
         //! \sa SCI_SETMARGINSENSITIVEN, SCN_MARGINCLICK()
         SCI_GETMARGINSENSITIVEN = 2247,
 
+        //! This message sets the cursor shape displayed over a margin.
+        //! \a wParam is the number of the margin.
+        //! \a lParam is the cursor shape, normally either SC_CURSORARROW or
+        //! SC_CURSORREVERSEARROW.  Note that, currently, QScintilla implements
+        //! both of these as Qt::ArrowCursor.
+        //!
+        //! \sa SCI_GETMARGINCURSORN
+        SCI_SETMARGINCURSORN = 2248,
+
+        //! This message returns the cursor shape displayed over a margin.
+        //! \a wParam is the number of the margin.
+        //!
+        //! \sa SCI_SETMARGINCURSORN
+        SCI_GETMARGINCURSORN = 2249,
+
         //!
         SCI_STYLECLEARALL = 2050,
 
@@ -463,6 +477,12 @@ public:
         //!
         SCI_SETWHITESPACEBACK = 2085,
 
+        //!
+        SCI_SETWHITESPACESIZE = 2086,
+
+        //!
+        SCI_GETWHITESPACESIZE = 2087,
+
         //!
         SCI_SETSTYLEBITS = 2090,
 
@@ -985,6 +1005,12 @@ public:
         //!
         SCI_SETFOLDMARGINHICOLOUR = 2291,
 
+        //!
+        SCI_MARKERSETBACKSELECTED = 2292,
+
+        //!
+        SCI_MARKERENABLEHIGHLIGHT = 2293,
+
         //!
         SCI_LINEDOWN = 2300,
 
@@ -1295,7 +1321,7 @@ public:
         //! ports of Scintilla this is a pointer to either raw or textual XPM
         //! image data.
         //!
-        //! \sa SCI_CLEARREGISTEREDIMAGES
+        //! \sa SCI_CLEARREGISTEREDIMAGES, SCI_REGISTERRGBAIMAGE
         SCI_REGISTERIMAGE = 2405,
 
         //!
@@ -1306,7 +1332,7 @@ public:
 
         //! This message de-registers all currently registered images.
         //!
-        //! \sa SCI_REGISTERIMAGE
+        //! \sa SCI_REGISTERIMAGE, SCI_REGISTERRGBAIMAGE
         SCI_CLEARREGISTEREDIMAGES = 2408,
 
         //!
@@ -1489,6 +1515,12 @@ public:
         //!
         SCI_GETCARETLINEBACKALPHA = 2471,
 
+        //!
+        SCI_SETWRAPINDENTMODE = 2472,
+
+        //!
+        SCI_GETWRAPINDENTMODE = 2473,
+
         //!
         SCI_MARKERSETALPHA = 2476,
 
@@ -1555,6 +1587,12 @@ public:
         //!
         SCI_GETHOTSPOTSINGLELINE = 2497,
 
+        //!
+        SCI_BRACEHIGHLIGHTINDICATOR = 2498,
+
+        //!
+        SCI_BRACEBADLIGHTINDICATOR = 2499,
+
         //!
         SCI_SETINDICATORCURRENT = 2500,
 
@@ -1674,6 +1712,9 @@ public:
         //!
         SCI_MARGINGETSTYLEOFFSET = 2538,
 
+        //!
+        SCI_SETMARGINOPTIONS = 2539,
+
         //!
         SCI_ANNOTATIONSETTEXT = 2540,
 
@@ -1710,9 +1751,246 @@ public:
         //!
         SCI_ANNOTATIONGETSTYLEOFFSET = 2551,
 
+        //!
+        SCI_SETEMPTYSELECTION = 2556,
+
+        //!
+        SCI_GETMARGINOPTIONS = 2557,
+
+        //!
+        SCI_INDICSETOUTLINEALPHA = 2558,
+
+        //!
+        SCI_INDICGETOUTLINEALPHA = 2559,
+
         //!
         SCI_ADDUNDOACTION = 2560,
 
+        //!
+        SCI_CHARPOSITIONFROMPOINT = 2561,
+
+        //!
+        SCI_CHARPOSITIONFROMPOINTCLOSE = 2562,
+
+        //!
+        SCI_SETMULTIPLESELECTION = 2563,
+
+        //!
+        SCI_GETMULTIPLESELECTION = 2564,
+
+        //!
+        SCI_SETADDITIONALSELECTIONTYPING = 2565,
+
+        //!
+        SCI_GETADDITIONALSELECTIONTYPING = 2566,
+
+        //!
+        SCI_SETADDITIONALCARETSBLINK = 2567,
+
+        //!
+        SCI_GETADDITIONALCARETSBLINK = 2568,
+
+        //!
+        SCI_SETADDITIONALCARETSVISIBLE = 2608,
+
+        //!
+        SCI_GETADDITIONALCARETSVISIBLE = 2609,
+
+        //!
+        SCI_AUTOCGETCURRENTTEXT = 2610,
+
+        //!
+        SCI_GETSELECTIONS = 2570,
+
+        //!
+        SCI_CLEARSELECTIONS = 2571,
+
+        //!
+        SCI_SETSELECTION = 2572,
+
+        //!
+        SCI_ADDSELECTION = 2573,
+
+        //!
+        SCI_SETMAINSELECTION = 2574,
+
+        //!
+        SCI_GETMAINSELECTION = 2575,
+
+        //!
+        SCI_SETSELECTIONNCARET = 2576,
+
+        //!
+        SCI_GETSELECTIONNCARET = 2577,
+
+        //!
+        SCI_SETSELECTIONNANCHOR = 2578,
+
+        //!
+        SCI_GETSELECTIONNANCHOR = 2579,
+
+        //!
+        SCI_SETSELECTIONNCARETVIRTUALSPACE = 2580,
+
+        //!
+        SCI_GETSELECTIONNCARETVIRTUALSPACE = 2581,
+
+        //!
+        SCI_SETSELECTIONNANCHORVIRTUALSPACE = 2582,
+
+        //!
+        SCI_GETSELECTIONNANCHORVIRTUALSPACE = 2583,
+
+        //!
+        SCI_SETSELECTIONNSTART = 2584,
+
+        //!
+        SCI_GETSELECTIONNSTART = 2585,
+
+        //!
+        SCI_SETSELECTIONNEND = 2586,
+
+        //!
+        SCI_GETSELECTIONNEND = 2587,
+
+        //!
+        SCI_SETRECTANGULARSELECTIONCARET = 2588,
+
+        //!
+        SCI_GETRECTANGULARSELECTIONCARET = 2589,
+
+        //!
+        SCI_SETRECTANGULARSELECTIONANCHOR = 2590,
+
+        //!
+        SCI_GETRECTANGULARSELECTIONANCHOR = 2591,
+
+        //!
+        SCI_SETRECTANGULARSELECTIONCARETVIRTUALSPACE = 2592,
+
+        //!
+        SCI_GETRECTANGULARSELECTIONCARETVIRTUALSPACE = 2593,
+
+        //!
+        SCI_SETRECTANGULARSELECTIONANCHORVIRTUALSPACE = 2594,
+
+        //!
+        SCI_GETRECTANGULARSELECTIONANCHORVIRTUALSPACE = 2595,
+
+        //!
+        SCI_SETVIRTUALSPACEOPTIONS = 2596,
+
+        //!
+        SCI_GETVIRTUALSPACEOPTIONS = 2597,
+
+        //!
+        SCI_SETRECTANGULARSELECTIONMODIFIER = 2598,
+
+        //!
+        SCI_GETRECTANGULARSELECTIONMODIFIER = 2599,
+
+        //!
+        SCI_SETADDITIONALSELFORE = 2600,
+
+        //!
+        SCI_SETADDITIONALSELBACK = 2601,
+
+        //!
+        SCI_SETADDITIONALSELALPHA = 2602,
+
+        //!
+        SCI_GETADDITIONALSELALPHA = 2603,
+
+        //!
+        SCI_SETADDITIONALCARETFORE = 2604,
+
+        //!
+        SCI_GETADDITIONALCARETFORE = 2605,
+
+        //!
+        SCI_ROTATESELECTION = 2606,
+
+        //!
+        SCI_SWAPMAINANCHORCARET = 2607,
+
+        //!
+        SCI_SETFONTQUALITY = 2611,
+
+        //!
+        SCI_GETFONTQUALITY = 2612,
+
+        //!
+        SCI_SETFIRSTVISIBLELINE = 2613,
+
+        //!
+        SCI_SETMULTIPASTE = 2614,
+
+        //!
+        SCI_GETMULTIPASTE = 2615,
+
+        //!
+        SCI_GETTAG = 2616,
+
+        //!
+        SCI_CHANGELEXERSTATE = 2617,
+
+        //!
+        SCI_CONTRACTEDFOLDNEXT = 2618,
+
+        //!
+        SCI_VERTICALCENTRECARET = 2619,
+
+        //!
+        SCI_MOVESELECTEDLINESUP = 2620,
+
+        //!
+        SCI_MOVESELECTEDLINESDOWN = 2621,
+
+        //!
+        SCI_SETIDENTIFIER = 2622,
+
+        //!
+        SCI_GETIDENTIFIER = 2623,
+
+        //! This message sets the width of an RGBA image specified by a future
+        //! call to SCI_MARKERDEFINERGBAIMAGE or SCI_REGISTERRGBAIMAGE.
+        //!
+        //! \sa SCI_RGBAIMAGESETHEIGHT, SCI_MARKERDEFINERGBAIMAGE,
+        //! SCI_REGISTERRGBAIMAGE.
+        SCI_RGBAIMAGESETWIDTH = 2624,
+
+        //! This message sets the height of an RGBA image specified by a future
+        //! call to SCI_MARKERDEFINERGBAIMAGE or SCI_REGISTERRGBAIMAGE.
+        //!
+        //! \sa SCI_RGBAIMAGESETWIDTH, SCI_MARKERDEFINERGBAIMAGE,
+        //! SCI_REGISTERRGBAIMAGE.
+        SCI_RGBAIMAGESETHEIGHT = 2625,
+
+        //! This message sets the symbol used to draw one of the 32 markers to
+        //! an RGBA image.  RGBA images use the SC_MARK_RGBAIMAGE marker
+        //! symbol.
+        //! \a wParam is the number of the marker.
+        //! \a lParam is a pointer to a QImage instance.  Note that in other
+        //! ports of Scintilla this is a pointer to raw RGBA image data.
+        //!
+        //! \sa SCI_MARKERDEFINE, SCI_MARKERDEFINEPIXMAP
+        SCI_MARKERDEFINERGBAIMAGE = 2626,
+
+        //! This message takes a copy of an image and registers it so that it
+        //! can be refered to by a unique integer identifier.
+        //! \a wParam is the image's identifier.
+        //! \a lParam is a pointer to a QImage instance.  Note that in other
+        //! ports of Scintilla this is a pointer to raw RGBA image data.
+        //!
+        //! \sa SCI_CLEARREGISTEREDIMAGES, SCI_REGISTERIMAGE
+        SCI_REGISTERRGBAIMAGE = 2627,
+
+        //!
+        SCI_SCROLLTOSTART = 2628,
+
+        //!
+        SCI_SCROLLTOEND = 2629,
+
         //!
         SCI_STARTRECORD = 3001,
 
@@ -1756,7 +2034,25 @@ public:
         SCI_GETPROPERTYINT = 4010,
 
         //!
-        SCI_GETSTYLEBITSNEEDED = 4011
+        SCI_GETSTYLEBITSNEEDED = 4011,
+
+        //!
+        SCI_GETLEXERLANGUAGE = 4012,
+
+        //!
+        SCI_PRIVATELEXERCALL = 4013,
+
+        //!
+        SCI_PROPERTYNAMES = 4014,
+
+        //!
+        SCI_PROPERTYTYPE = 4015,
+
+        //!
+        SCI_DESCRIBEPROPERTY = 4016,
+
+        //!
+        SCI_DESCRIBEKEYWORDSETS = 4017
     };
 
     enum
@@ -1766,6 +2062,49 @@ public:
         SC_ALPHA_NOALPHA = 256
     };
 
+    enum
+    {
+        SC_CARETSTICKY_OFF = 0,
+        SC_CARETSTICKY_ON = 1,
+        SC_CARETSTICKY_WHITESPACE = 2
+    };
+
+    enum
+    {
+        SC_EFF_QUALITY_MASK = 0x0f,
+        SC_EFF_QUALITY_DEFAULT = 0,
+        SC_EFF_QUALITY_NON_ANTIALIASED = 1,
+        SC_EFF_QUALITY_ANTIALIASED = 2,
+        SC_EFF_QUALITY_LCD_OPTIMIZED = 3
+    };
+
+    enum
+    {
+        SC_MARGINOPTION_NONE = 0x00,
+        SC_MARGINOPTION_SUBLINESELECT = 0x01
+    };
+
+    enum
+    {
+        SC_MULTIPASTE_ONCE = 0,
+        SC_MULTIPASTE_EACH = 1
+    };
+
+    enum
+    {
+        SC_TYPE_BOOLEAN = 0,
+        SC_TYPE_INTEGER = 1,
+        SC_TYPE_STRING = 2
+    };
+
+    enum
+    {
+        SC_UPDATE_CONTENT = 0x01,
+        SC_UPDATE_SELECTION = 0x02,
+        SC_UPDATE_V_SCROLL = 0x04,
+        SC_UPDATE_H_SCROLL = 0x08
+    };
+
     enum
     {
         SC_WRAPVISUALFLAG_NONE = 0x0000,
@@ -1787,7 +2126,15 @@ public:
     {
         SC_SEL_STREAM = 0,
         SC_SEL_RECTANGLE = 1,
-        SC_SEL_LINES = 2
+        SC_SEL_LINES = 2,
+        SC_SEL_THIN = 3
+    };
+
+    enum
+    {
+        SCVS_NONE = 0,
+        SCVS_RECTANGULARSELECTION = 1,
+        SCVS_USERACCESSIBLE = 2
     };
 
     enum
@@ -1885,8 +2232,8 @@ public:
         //! A drawn minus sign in a connected circle.
         SC_MARK_CIRCLEMINUSCONNECTED = 21,
 
-        //! No symbol is drawn but the line of text is drawn with the
-        //! same background colour.
+        //! No symbol is drawn but the line is drawn with the same background
+        //! color as the marker's.
         SC_MARK_BACKGROUND = 22,
 
         //! Three drawn dots.
@@ -1898,15 +2245,23 @@ public:
         //! An XPM format pixmap.
         SC_MARK_PIXMAP = 25,
 
-        //! A full rectangle (ie. the margin background).
+        //! A full rectangle (ie. the margin background) using the marker's
+        //! background color.
         SC_MARK_FULLRECT = 26,
 
-        //! A left rectangle (ie. part of the margin background).
+        //! A left rectangle (ie. the left part of the margin background) using
+        //! the marker's background color.
         SC_MARK_LEFTRECT = 27,
 
         //! The value is available for plugins to use.
         SC_MARK_AVAILABLE = 28,
 
+        //! The line is underlined using the marker's background color.
+        SC_MARK_UNDERLINE = 29,
+
+        //! A RGBA format image.
+        SC_MARK_RGBAIMAGE = 30,
+
         //! Characters can be used as symbols by adding this to the ASCII value
         //! of the character.
         SC_MARK_CHARACTER = 10000
@@ -2021,7 +2376,6 @@ public:
 
     enum
     {
-        INDIC_MAX = 31,
         INDIC_PLAIN = 0,
         INDIC_SQUIGGLE = 1,
         INDIC_TT = 2,
@@ -2030,7 +2384,14 @@ public:
         INDIC_HIDDEN = 5,
         INDIC_BOX = 6,
         INDIC_ROUNDBOX = 7,
+        INDIC_STRAIGHTBOX = 8,
+        INDIC_DASH = 9,
+        INDIC_DOTS = 10,
+        INDIC_SQUIGGLELOW = 11,
+        INDIC_DOTBOX = 12,
+
         INDIC_CONTAINER = 8,
+        INDIC_MAX = 31,
         INDIC0_MASK = 0x20,
         INDIC1_MASK = 0x40,
         INDIC2_MASK = 0x80,
@@ -2084,6 +2445,13 @@ public:
         SC_WRAP_CHAR = 2
     };
 
+    enum
+    {
+        SC_WRAPINDENT_FIXED = 0,
+        SC_WRAPINDENT_SAME = 1,
+        SC_WRAPINDENT_INDENT = 2
+    };
+
     enum
     {
         SC_CACHE_NONE = 0,
@@ -2109,7 +2477,9 @@ public:
     enum
     {
         SC_CURSORNORMAL = -1,
-        SC_CURSORWAIT = 4
+        SC_CURSORARROW = 2,
+        SC_CURSORWAIT = 4,
+        SC_CURSORREVERSEARROW = 7
     };
 
     enum
@@ -2159,7 +2529,8 @@ public:
         SC_MOD_CHANGEMARGIN = 0x10000,
         SC_MOD_CHANGEANNOTATION = 0x20000,
         SC_MOD_CONTAINER = 0x40000,
-        SC_MODEVENTMASKALL = 0x7ffff
+        SC_MOD_LEXERSTATE = 0x80000,
+        SC_MODEVENTMASKALL = 0xfffff
     };
 
     enum
@@ -2195,11 +2566,19 @@ public:
         //! Shift key.
         SCMOD_SHIFT = 1,
 
-        //! Control key.
+        //! Control key (the Command key on OS/X, the Ctrl key on other
+        //! platforms).
         SCMOD_CTRL = 2,
 
         //! Alt key.
-        SCMOD_ALT = 4
+        SCMOD_ALT = 4,
+
+        //! This is the same as SCMOD_META on all platforms.
+        SCMOD_SUPER = 8,
+
+        //! Meta key (the Ctrl key on OS/X, the Windows key on other
+        //! platforms).
+        SCMOD_META = 16
     };
 
     //! This enum defines the different language lexers.
@@ -2498,7 +2877,19 @@ public:
         SCLEX_NIMROD = 96,
 
         //! Select the SML lexer.
-        SCLEX_SML = 97
+        SCLEX_SML = 97,
+
+        //! Select the Markdown lexer.
+        SCLEX_MARKDOWN = 98,
+
+        //! Select the txt2tags lexer.
+        SCLEX_TXT2TAGS = 99,
+
+        //! Select the 68000 assembler lexer.
+        SCLEX_A68K = 100,
+
+        //! Select the Modula 3 lexer.
+        SCLEX_MODULA = 101
     };
 
     //! Construct an empty QsciScintillaBase with parent \a parent, name \a
@@ -2558,6 +2949,10 @@ public:
     long SendScintilla(unsigned int msg, unsigned long wParam,
             const QPixmap &lParam) const;
 
+    //! \overload
+    long SendScintilla(unsigned int msg, unsigned long wParam,
+            const QImage &lParam) const;
+
     //! Send the Scintilla message \a msg and return a pointer result.
     void *SendScintillaPtrResult(unsigned int msg) const;
 
@@ -2640,6 +3035,13 @@ signals:
     //! when the user double clicked.
     void SCN_HOTSPOTDOUBLECLICK(int position, int modifiers);
 
+    //! This signal is emitted when the user releases the mouse button on text
+    //! in a style with the hotspot attribute set.
+    //! \a position is the position in the text where the release occured.
+    //! \a modifiers is the logical or of the modifier keys that were pressed
+    //! when the user released the button.
+    void SCN_HOTSPOTRELEASECLICK(int position, int modifiers);
+
     //! This signal is emitted when the user clicks on text that has an
     //! indicator.
     //! \a position is the position in the text where the click occured.
@@ -2705,8 +3107,11 @@ signals:
     //! \sa SCI_COLOURISE, SCI_GETENDSTYLED
     void SCN_STYLENEEDED(int position);
 
-    //!
-    void SCN_UPDATEUI();
+    //! This signal is emitted when either the text or styling of the text has
+    //! changed or the selection range or scroll position has changed.
+    //! \a updated contains the set of SC_UPDATE_* flags describing the changes
+    //! since the signal was last emitted.
+    void SCN_UPDATEUI(int updated);
 
     //!
     void SCN_USERLISTSELECTION(const char *, int);
