@@ -1,6 +1,6 @@
 // This module implements the "official" low-level API.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -361,6 +356,14 @@ long QsciScintillaBase::SendScintilla(unsigned int msg, unsigned long wParam,
 }
 
 
+// Overloaded message send.
+long QsciScintillaBase::SendScintilla(unsigned int msg, unsigned long wParam,
+        const QImage &lParam) const
+{
+    return sci->WndProc(msg, wParam, reinterpret_cast<sptr_t>(&lParam));
+}
+
+
 // Send a message to the real Scintilla widget using the low level Scintilla
 // API that returns a pointer result.
 void *QsciScintillaBase::SendScintillaPtrResult(unsigned int msg) const
@@ -419,12 +422,20 @@ void QsciScintillaBase::handleSelection()
 // Handle key presses.
 void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
 {
-    unsigned key;
+    unsigned key, modifiers = 0;
     QCString utf8;
 
-    bool shift = e->state() & Qt::ShiftButton;
-    bool ctrl = e->state() & Qt::ControlButton;
-    bool alt = e->state() & Qt::AltButton;
+    if (e->state() & Qt::ShiftButton)
+        modifiers |= SCMOD_SHIFT;
+
+    if (e->state() & Qt::ControlButton)
+        modifiers |= SCMOD_CTRL;
+
+    if (e->state() & Qt::AltButton)
+        modifiers |= SCMOD_ALT;
+
+    if (e->state() & Qt::MetaButton)
+        modifiers |= SCMOD_META;
 
     switch (e->key())
     {
@@ -480,6 +491,12 @@ void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
         key = SCK_TAB;
         break;
 
+    case Qt::Key_Backtab:
+        // Scintilla assumes a backtab is shift-tab.
+        key = SCK_TAB;
+        modifiers |= SCMOD_SHIFT;
+        break;
+
     case Qt::Key_Return:
     case Qt::Key_Enter:
         key = SCK_RETURN;
@@ -499,9 +516,9 @@ void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
 
     default:
         // See if the input was a single ASCII key.  If so it will be passed to
-        // KeyDown to allow it to be filtered.  Correct the modifiers and key
-        // for ASCII letters as Qt uses the ASCII code of uppercase letters for
-        // Key_A etc.
+        // KeyDownWithModifiers to allow it to be filtered.  Correct the
+        // modifiers and key for ASCII letters as Qt uses the ASCII code of
+        // uppercase letters for Key_A etc.
         utf8 = e->text().utf8();
 
         if (utf8.length() == 0)
@@ -513,11 +530,11 @@ void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
         else if (key >= 0x01 && key <= 0x1a)
             key += 0x40;
         else if (key >= 'A' && key <= 'Z')
-            shift = true;
+            modifiers |= true;
         else if (key >= 'a' && key <= 'z')
         {
             key -= 0x20;
-            shift = false;
+            modifiers &= ~SCMOD_SHIFT;
         }
     }
 
@@ -525,7 +542,7 @@ void QsciScintillaBase::keyPressEvent(QKeyEvent *e)
     {
         bool consumed = false;
 
-        sci->KeyDown(key, shift, ctrl, alt, &consumed);
+        sci->KeyDownWithModifiers(key, modifiers, &consumed);
 
         if (consumed)
         {
@@ -602,9 +619,19 @@ void QsciScintillaBase::mousePressEvent(QMouseEvent *e)
 
         triple_click.stop();
 
+        // Scintilla uses the Alt modifier to initiate rectangular selection.
+        // However the GTK port (under X11, not Windows) uses the Control
+        // modifier (by default, although it is configurable).  It does this
+        // because most X11 window managers hijack Alt-drag to move the window.
+        // We do the same, except that (for the moment at least) we don't allow
+        // the modifier to be configured.
         bool shift = e->state() & Qt::ShiftButton;
         bool ctrl = e->state() & Qt::ControlButton;
-        bool alt = e->state() & Qt::AltButton;
+#if defined(Q_WS_X11)
+        bool alt = ctrl;
+#else
+        bool alt = e->modifiers() & Qt::AltButton;
+#endif
 
         sci->ButtonDown(pt, clickTime, shift, ctrl, alt);
     }
@@ -673,15 +700,16 @@ void QsciScintillaBase::dragEnterEvent(QDragEnterEvent *e)
 // Handle drag leaves.
 void QsciScintillaBase::dragLeaveEvent(QDragLeaveEvent *)
 {
-    sci->SetDragPosition(-1);
+    sci->SetDragPosition(SelectionPosition());
 }
 
 
 // Handle drag moves.
 void QsciScintillaBase::dragMoveEvent(QDragMoveEvent *e)
 {
-    sci->SetDragPosition(sci->PositionFromLocation(Point(e->pos().x(),
-                    e->pos().y())));
+    sci->SetDragPosition(
+            sci->SPositionFromLocation(Point(e->pos().x(), e->pos().y()),
+                    false, false, sci->UserVirtualSpace()));
 
     if (sci->pdoc->IsReadOnly() || !QTextDrag::canDecode(e))
     {
@@ -698,6 +726,7 @@ void QsciScintillaBase::dropEvent(QDropEvent *e)
 {
     bool moving;
     const char *s;
+    bool rectangular;
 
     QString text;
 
@@ -719,9 +748,13 @@ void QsciScintillaBase::dropEvent(QDropEvent *e)
         s = cs.data();
     }
     else
+    {
         s = text.latin1();
+    }
 
-    sci->DropAt(sci->posDrop, s, moving, false);
+    rectangular = false;
+
+    sci->DropAt(sci->posDrop, s, moving, rectangular);
     sci->Redraw();
 }
 
