@@ -1,6 +1,6 @@
 // This module implements the portability layer for the Qt port of Scintilla.
 //
-// Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,18 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
-// 
-// This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-// INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-// granted herein.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -57,21 +47,23 @@
 #include "Qsci/qsciscintillabase.h"
 #include "SciClasses.h"
 
+#include "FontQuality.h"
+
 
 // Type convertors.
-static QFont *PFont(FontID id)
+static QFont *PFont(FontID fid)
 {
-    return reinterpret_cast<QFont *>(id);
+    return reinterpret_cast<QFont *>(fid);
 }
 
-static QWidget *PWindow(WindowID id)
+static QWidget *PWindow(WindowID wid)
 {
-    return reinterpret_cast<QWidget *>(id);
+    return reinterpret_cast<QWidget *>(wid);
 }
 
-static SciPopup *PMenu(MenuID id)
+static SciPopup *PMenu(MenuID mid)
 {
-    return reinterpret_cast<SciPopup *>(id);
+    return reinterpret_cast<SciPopup *>(mid);
 }
 
 
@@ -113,7 +105,7 @@ void Palette::Allocate(Window &)
 
 
 // Font management.
-Font::Font() : id(0)
+Font::Font() : fid(0)
 {
 }
 
@@ -122,15 +114,43 @@ Font::~Font()
 }
 
 void Font::Create(const char *faceName, int, int size, bool bold, bool italic,
-        bool)
+        int flags)
 {
     Release();
 
     QFont *f = new QFont();
 
+    QFont::StyleStrategy strategy;
+
+    switch (flags & SC_EFF_QUALITY_MASK)
+    {
+    case SC_EFF_QUALITY_NON_ANTIALIASED:
+        strategy = QFont::NoAntialias;
+        break;
+
+    case SC_EFF_QUALITY_ANTIALIASED:
+        strategy = QFont::PreferAntialias;
+        break;
+
+    default:
+        strategy = QFont::PreferDefault;
+    }
+
+#if defined(Q_WS_MAC)
+#if QT_VERSION >= 0x040700
+    strategy = static_cast<QFont::StyleStrategy>(strategy | QFont::ForceIntegerMetrics);
+#else
+#warning "Correct handling of QFont metrics requires Qt v4.7.0 or later"
+#endif
+#endif
+
+    f->setStyleStrategy(strategy);
+
     // If name of the font begins with a '-', assume, that it is an XLFD.
     if (faceName[0] == '-')
+    {
         f->setRawName(faceName);
+    }
     else
     {
         f->setFamily(faceName);
@@ -139,15 +159,15 @@ void Font::Create(const char *faceName, int, int size, bool bold, bool italic,
         f->setItalic(italic);
     }
 
-    id = f;
+    fid = f;
 }
 
 void Font::Release()
 {
-    if (id)
+    if (fid)
     {
-        delete PFont(id);
-        id = 0;
+        delete PFont(fid);
+        fid = 0;
     }
 }
 
@@ -182,6 +202,7 @@ public:
     void AlphaRectangle(PRectangle rc, int cornerSize, ColourAllocated fill,
             int alphaFill, ColourAllocated outline, int alphaOutline,
             int flags);
+    void DrawRGBAImage(PRectangle rc, int width, int height, const unsigned char *pixelsImage);
     void Ellipse(PRectangle rc, ColourAllocated fore, ColourAllocated back);
     void Copy(PRectangle rc, Point from, Surface &surfaceSource);
 
@@ -215,9 +236,8 @@ private:
             int len, ColourAllocated fore);
     QFontMetrics metrics(Font &font_);
     QString convertText(const char *s, int len);
-    static QRgb convertQRgb(const ColourAllocated &col, unsigned alpha);
     static QColor convertQColor(const ColourAllocated &col,
-            unsigned alpha = 0xff);
+            unsigned alpha = 255);
 
     bool unicodeMode;
     QPaintDevice *pd;
@@ -398,11 +418,11 @@ void SurfaceImpl::AlphaRectangle(PRectangle rc, int cornerSize,
 
     // Assume that "cornerSize" means outline width.
     if (cornerSize > 0)
-        painter->setPen(QPen(QColor(convertQRgb(outline, alphaOutline)), cornerSize));
+        painter->setPen(QPen(convertQColor(outline, alphaOutline), cornerSize));
     else
         painter->setPen(Qt::NoPen);
 
-    painter->setBrush(QColor(convertQRgb(fill, alphaFill)));
+    painter->setBrush(convertQColor(fill, alphaFill));
     painter->drawRect(rc.left, rc.top, w, h);
 }
 
@@ -490,6 +510,20 @@ void SurfaceImpl::DrawXPM(PRectangle rc, const XPM *xpm)
     painter->drawPixmap(x, y, qpm);
 }
 
+void SurfaceImpl::DrawRGBAImage(PRectangle rc, int width, int height,
+        const unsigned char *pixelsImage)
+{
+    Q_ASSERT(painter);
+
+    int x, y;
+    const QImage *qim = reinterpret_cast<const QImage *>(pixelsImage);
+
+    x = rc.left + (rc.Width() - width) / 2;
+    y = rc.top + (rc.Height() - height) / 2;
+
+    painter->drawImage(x, y, *qim);
+}
+
 void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len,
         int *positions)
 {
@@ -502,7 +536,9 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len,
 
     for (int i_char = 0; i_char < qs.length(); ++i_char)
     {
-        int width = fm.width(qs, i_char + 1);
+        // We can't just add the individual character widths together because
+        // of kerning.
+        int width = fm.width(qs.left(i_char + 1));
 
         if (unicodeMode)
         {
@@ -589,8 +625,9 @@ QString SurfaceImpl::convertText(const char *s, int len)
     return QString::fromLatin1(s, len);
 }
 
-// Convert a Scintilla colour and alpha component to a Qt QRgb.
-QRgb SurfaceImpl::convertQRgb(const ColourAllocated &col, unsigned alpha)
+
+// Convert a Scintilla colour, and alpha component, to a Qt QColor.
+QColor SurfaceImpl::convertQColor(const ColourAllocated &col, unsigned alpha)
 {
     long c = col.AsLong();
 
@@ -598,15 +635,7 @@ QRgb SurfaceImpl::convertQRgb(const ColourAllocated &col, unsigned alpha)
     unsigned g = (c >> 8) & 0xff;
     unsigned b = (c >> 16) & 0xff;
 
-    QRgb rgba = (alpha << 24) | (r << 16) | (g << 8) | b;
-
-    return rgba;
-}
-
-// Convert a Scintilla colour, and optional alpha component, to a Qt QColor.
-QColor SurfaceImpl::convertQColor(const ColourAllocated &col, unsigned alpha)
-{
-    return QColor(convertQRgb(col, alpha));
+    return QColor(r, g, b, alpha);
 }
 
 
@@ -617,7 +646,7 @@ Window::~Window()
 
 void Window::Destroy()
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     if (w)
     {
@@ -628,18 +657,18 @@ void Window::Destroy()
         // around the problem but this is the simplest and doesn't seem to
         // cause problems of its own.
         w->deleteLater();
-        id = 0;
+        wid = 0;
     }
 }
 
 bool Window::HasFocus()
 {
-    return PWindow(id)->hasFocus();
+    return PWindow(wid)->hasFocus();
 }
 
 PRectangle Window::GetPosition()
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     // Before any size allocated pretend its big enough not to be scrolled.
     PRectangle rc(0,0,5000,5000);
@@ -657,19 +686,19 @@ PRectangle Window::GetPosition()
 
 void Window::SetPosition(PRectangle rc)
 {
-    PWindow(id)->setGeometry(rc.left, rc.top, rc.right - rc.left,
+    PWindow(wid)->setGeometry(rc.left, rc.top, rc.right - rc.left,
             rc.bottom - rc.top);
 }
 
 void Window::SetPositionRelative(PRectangle rc, Window relativeTo)
 {
-    QWidget *rel = PWindow(relativeTo.id);
+    QWidget *rel = PWindow(relativeTo.wid);
     QPoint pos = rel->mapToGlobal(rel->pos());
 
     int x = pos.x() + rc.left;
     int y = pos.y() + rc.top;
 
-    PWindow(id)->setGeometry(x, y, rc.right - rc.left, rc.bottom - rc.top);
+    PWindow(wid)->setGeometry(x, y, rc.right - rc.left, rc.bottom - rc.top);
 }
 
 PRectangle Window::GetClientPosition()
@@ -679,7 +708,7 @@ PRectangle Window::GetClientPosition()
 
 void Window::Show(bool show)
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     if (show)
         w->show();
@@ -689,7 +718,7 @@ void Window::Show(bool show)
 
 void Window::InvalidateAll()
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     if (w)
         w->update();
@@ -697,7 +726,7 @@ void Window::InvalidateAll()
 
 void Window::InvalidateRectangle(PRectangle rc)
 {
-    QWidget *w = PWindow(id);
+    QWidget *w = PWindow(wid);
 
     if (w)
         w->update(rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top);
@@ -705,7 +734,7 @@ void Window::InvalidateRectangle(PRectangle rc)
 
 void Window::SetFont(Font &font)
 {
-    PWindow(id)->setFont(*PFont(font.GetID()));
+    PWindow(wid)->setFont(*PFont(font.GetID()));
 }
 
 void Window::SetCursor(Cursor curs)
@@ -739,53 +768,55 @@ void Window::SetCursor(Cursor curs)
         break;
 
     default:
+        // Note that Qt doesn't have a standard cursor that could be used to
+        // implement cursorReverseArrow.
         qc = Qt::ArrowCursor;
     }
 
-    PWindow(id)->setCursor(qc);
+    PWindow(wid)->setCursor(qc);
 }
 
 void Window::SetTitle(const char *s)
 {
-    PWindow(id)->setWindowTitle(s);
+    PWindow(wid)->setWindowTitle(s);
 }
 
 
 PRectangle Window::GetMonitorRect(Point pt)
 {
-    QPoint qpt = PWindow(id)->mapToGlobal(QPoint(pt.x, pt.y));
+    QPoint qpt = PWindow(wid)->mapToGlobal(QPoint(pt.x, pt.y));
     QRect qr = QApplication::desktop()->availableGeometry(qpt);
-    qpt = PWindow(id)->mapFromGlobal(qr.topLeft());
+    qpt = PWindow(wid)->mapFromGlobal(qr.topLeft());
 
     return PRectangle(qpt.x(), qpt.y(), qpt.x() + qr.width(), qpt.y() + qr.height());
 }
 
 
 // Menu management.
-Menu::Menu() : id(0)
+Menu::Menu() : mid(0)
 {
 }
 
 void Menu::CreatePopUp()
 {
     Destroy();
-    id = new SciPopup();
+    mid = new SciPopup();
 }
 
 void Menu::Destroy()
 {
-    SciPopup *m = PMenu(id);
+    SciPopup *m = PMenu(mid);
 
     if (m)
     {
         delete m;
-        id = 0;
+        mid = 0;
     }
 }
 
 void Menu::Show(Point pt, Window &)
 {
-    PMenu(id)->popup(QPoint(pt.x, pt.y));
+    PMenu(mid)->popup(QPoint(pt.x, pt.y));
 }
 
 
