@@ -1,6 +1,6 @@
 // This class defines the "official" low-level API.
 //
-// Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,18 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
-// 
-// This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-// INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-// granted herein.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -50,9 +40,13 @@ extern "C++" {
 #include <Qsci/qsciglobal.h>
 
 
+QT_BEGIN_NAMESPACE
 class QColor;
+class QImage;
 class QPainter;
 class QPixmap;
+class QMimeData;
+QT_END_NAMESPACE
 
 class ScintillaQt;
 
@@ -238,7 +232,8 @@ public:
         //! \a wParam is the number of the marker.
         //! \a lParam is the marker symbol and is one of the SC_MARK_* values.
         //!
-        //! \sa SCI_MARKERADD, SCI_MARKERDEFINEPIXMAP
+        //! \sa SCI_MARKERADD, SCI_MARKERDEFINEPIXMAP,
+        //! SCI_MARKERDEFINERGBAIMAGE
         SCI_MARKERDEFINE = 2040,
 
         //! This message sets the foreground colour used to draw a marker.  A
@@ -314,31 +309,31 @@ public:
         //! ports of Scintilla this is a pointer to either raw or textual XPM
         //! image data.
         //!
-        //! \sa SCI_MARKERDEFINE
+        //! \sa SCI_MARKERDEFINE, SCI_MARKERDEFINERGBAIMAGE
         SCI_MARKERDEFINEPIXMAP = 2049,
 
         //! This message sets what can be displayed in a margin.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //! \a lParam is the logical or of the SC_MARGIN_* values.
         //!
         //! \sa SCI_GETMARGINTYPEN
         SCI_SETMARGINTYPEN = 2240,
 
         //! This message returns what can be displayed in a margin.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //!
         //! \sa SCI_SETMARGINTYPEN
         SCI_GETMARGINTYPEN = 2241,
 
         //! This message sets the width of a margin in pixels.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //! \a lParam is the new margin width.
         //!
         //! \sa SCI_GETMARGINWIDTHN
         SCI_SETMARGINWIDTHN = 2242,
 
         //! This message returns the width of a margin in pixels.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //!
         //! \sa SCI_SETMARGINWIDTHN
         SCI_GETMARGINWIDTHN = 2243,
@@ -347,20 +342,20 @@ public:
         //! with one bit for each possible marker.  If a bit is set then the
         //! corresponding marker is displayed.  By default, all markers are
         //! displayed.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //! \a lParam is the new margin mask.
         //!
         //! \sa SCI_GETMARGINMASKN, SCI_MARKERDEFINE
         SCI_SETMARGINMASKN = 2244,
 
         //! This message returns the mask of a margin.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //!
         //! \sa SCI_SETMARGINMASKN
         SCI_GETMARGINMASKN = 2245,
 
         //! This message sets the sensitivity of a margin to mouse clicks.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //! \a lParam is non-zero to make the margin sensitive to mouse clicks.
         //! When the mouse is clicked the SCN_MARGINCLICK() signal is emitted.
         //!
@@ -368,11 +363,26 @@ public:
         SCI_SETMARGINSENSITIVEN = 2246,
 
         //! This message returns the sensitivity of a margin to mouse clicks.
-        //! \a wParam is the number of the margin: 0, 1 or 2.
+        //! \a wParam is the number of the margin.
         //!
         //! \sa SCI_SETMARGINSENSITIVEN, SCN_MARGINCLICK()
         SCI_GETMARGINSENSITIVEN = 2247,
 
+        //! This message sets the cursor shape displayed over a margin.
+        //! \a wParam is the number of the margin.
+        //! \a lParam is the cursor shape, normally either SC_CURSORARROW or
+        //! SC_CURSORREVERSEARROW.  Note that, currently, QScintilla implements
+        //! both of these as Qt::ArrowCursor.
+        //!
+        //! \sa SCI_GETMARGINCURSORN
+        SCI_SETMARGINCURSORN = 2248,
+
+        //! This message returns the cursor shape displayed over a margin.
+        //! \a wParam is the number of the margin.
+        //!
+        //! \sa SCI_SETMARGINCURSORN
+        SCI_GETMARGINCURSORN = 2249,
+
         //!
         SCI_STYLECLEARALL = 2050,
 
@@ -466,6 +476,12 @@ public:
         //!
         SCI_SETWHITESPACEBACK = 2085,
 
+        //!
+        SCI_SETWHITESPACESIZE = 2086,
+
+        //!
+        SCI_GETWHITESPACESIZE = 2087,
+
         //!
         SCI_SETSTYLEBITS = 2090,
 
@@ -988,6 +1004,12 @@ public:
         //!
         SCI_SETFOLDMARGINHICOLOUR = 2291,
 
+        //!
+        SCI_MARKERSETBACKSELECTED = 2292,
+
+        //!
+        SCI_MARKERENABLEHIGHLIGHT = 2293,
+
         //!
         SCI_LINEDOWN = 2300,
 
@@ -1298,7 +1320,7 @@ public:
         //! ports of Scintilla this is a pointer to either raw or textual XPM
         //! image data.
         //!
-        //! \sa SCI_CLEARREGISTEREDIMAGES
+        //! \sa SCI_CLEARREGISTEREDIMAGES, SCI_REGISTERRGBAIMAGE
         SCI_REGISTERIMAGE = 2405,
 
         //!
@@ -1309,7 +1331,7 @@ public:
 
         //! This message de-registers all currently registered images.
         //!
-        //! \sa SCI_REGISTERIMAGE
+        //! \sa SCI_REGISTERIMAGE, SCI_REGISTERRGBAIMAGE
         SCI_CLEARREGISTEREDIMAGES = 2408,
 
         //!
@@ -1492,6 +1514,12 @@ public:
         //!
         SCI_GETCARETLINEBACKALPHA = 2471,
 
+        //!
+        SCI_SETWRAPINDENTMODE = 2472,
+
+        //!
+        SCI_GETWRAPINDENTMODE = 2473,
+
         //!
         SCI_MARKERSETALPHA = 2476,
 
@@ -1558,6 +1586,12 @@ public:
         //!
         SCI_GETHOTSPOTSINGLELINE = 2497,
 
+        //!
+        SCI_BRACEHIGHLIGHTINDICATOR = 2498,
+
+        //!
+        SCI_BRACEBADLIGHTINDICATOR = 2499,
+
         //!
         SCI_SETINDICATORCURRENT = 2500,
 
@@ -1615,6 +1649,347 @@ public:
         //!
         SCI_DELWORDRIGHTEND = 2518,
 
+        //! This message copies the selection.  If the selection is empty then
+        //! copy the line with the caret.
+        SCI_COPYALLOWLINE = 2519,
+
+        //! This message returns a pointer to the document text.  Any
+        //! subsequent message will invalidate the pointer.
+        SCI_GETCHARACTERPOINTER = 2520,
+
+        //!
+        SCI_SETKEYSUNICODE = 2521,
+
+        //!
+        SCI_GETKEYSUNICODE = 2522,
+
+        //!
+        SCI_INDICSETALPHA = 2523,
+
+        //!
+        SCI_INDICGETALPHA = 2524,
+
+        //!
+        SCI_SETEXTRAASCENT = 2525,
+
+        //!
+        SCI_GETEXTRAASCENT = 2526,
+
+        //!
+        SCI_SETEXTRADESCENT = 2527,
+
+        //!
+        SCI_GETEXTRADESCENT = 2528,
+
+        //!
+        SCI_MARKERSYMBOLDEFINED = 2529,
+
+        //!
+        SCI_MARGINSETTEXT = 2530,
+
+        //!
+        SCI_MARGINGETTEXT = 2531,
+
+        //!
+        SCI_MARGINSETSTYLE = 2532,
+
+        //!
+        SCI_MARGINGETSTYLE = 2533,
+
+        //!
+        SCI_MARGINSETSTYLES = 2534,
+
+        //!
+        SCI_MARGINGETSTYLES = 2535,
+
+        //!
+        SCI_MARGINTEXTCLEARALL = 2536,
+
+        //!
+        SCI_MARGINSETSTYLEOFFSET = 2537,
+
+        //!
+        SCI_MARGINGETSTYLEOFFSET = 2538,
+
+        //!
+        SCI_SETMARGINOPTIONS = 2539,
+
+        //!
+        SCI_ANNOTATIONSETTEXT = 2540,
+
+        //!
+        SCI_ANNOTATIONGETTEXT = 2541,
+
+        //!
+        SCI_ANNOTATIONSETSTYLE = 2542,
+
+        //!
+        SCI_ANNOTATIONGETSTYLE = 2543,
+
+        //!
+        SCI_ANNOTATIONSETSTYLES = 2544,
+
+        //!
+        SCI_ANNOTATIONGETSTYLES = 2545,
+
+        //!
+        SCI_ANNOTATIONGETLINES = 2546,
+
+        //!
+        SCI_ANNOTATIONCLEARALL = 2547,
+
+        //!
+        SCI_ANNOTATIONSETVISIBLE = 2548,
+
+        //!
+        SCI_ANNOTATIONGETVISIBLE = 2549,
+
+        //!
+        SCI_ANNOTATIONSETSTYLEOFFSET = 2550,
+
+        //!
+        SCI_ANNOTATIONGETSTYLEOFFSET = 2551,
+
+        //!
+        SCI_SETEMPTYSELECTION = 2556,
+
+        //!
+        SCI_GETMARGINOPTIONS = 2557,
+
+        //!
+        SCI_INDICSETOUTLINEALPHA = 2558,
+
+        //!
+        SCI_INDICGETOUTLINEALPHA = 2559,
+
+        //!
+        SCI_ADDUNDOACTION = 2560,
+
+        //!
+        SCI_CHARPOSITIONFROMPOINT = 2561,
+
+        //!
+        SCI_CHARPOSITIONFROMPOINTCLOSE = 2562,
+
+        //!
+        SCI_SETMULTIPLESELECTION = 2563,
+
+        //!
+        SCI_GETMULTIPLESELECTION = 2564,
+
+        //!
+        SCI_SETADDITIONALSELECTIONTYPING = 2565,
+
+        //!
+        SCI_GETADDITIONALSELECTIONTYPING = 2566,
+
+        //!
+        SCI_SETADDITIONALCARETSBLINK = 2567,
+
+        //!
+        SCI_GETADDITIONALCARETSBLINK = 2568,
+
+        //!
+        SCI_SETADDITIONALCARETSVISIBLE = 2608,
+
+        //!
+        SCI_GETADDITIONALCARETSVISIBLE = 2609,
+
+        //!
+        SCI_AUTOCGETCURRENTTEXT = 2610,
+
+        //!
+        SCI_GETSELECTIONS = 2570,
+
+        //!
+        SCI_CLEARSELECTIONS = 2571,
+
+        //!
+        SCI_SETSELECTION = 2572,
+
+        //!
+        SCI_ADDSELECTION = 2573,
+
+        //!
+        SCI_SETMAINSELECTION = 2574,
+
+        //!
+        SCI_GETMAINSELECTION = 2575,
+
+        //!
+        SCI_SETSELECTIONNCARET = 2576,
+
+        //!
+        SCI_GETSELECTIONNCARET = 2577,
+
+        //!
+        SCI_SETSELECTIONNANCHOR = 2578,
+
+        //!
+        SCI_GETSELECTIONNANCHOR = 2579,
+
+        //!
+        SCI_SETSELECTIONNCARETVIRTUALSPACE = 2580,
+
+        //!
+        SCI_GETSELECTIONNCARETVIRTUALSPACE = 2581,
+
+        //!
+        SCI_SETSELECTIONNANCHORVIRTUALSPACE = 2582,
+
+        //!
+        SCI_GETSELECTIONNANCHORVIRTUALSPACE = 2583,
+
+        //!
+        SCI_SETSELECTIONNSTART = 2584,
+
+        //!
+        SCI_GETSELECTIONNSTART = 2585,
+
+        //!
+        SCI_SETSELECTIONNEND = 2586,
+
+        //!
+        SCI_GETSELECTIONNEND = 2587,
+
+        //!
+        SCI_SETRECTANGULARSELECTIONCARET = 2588,
+
+        //!
+        SCI_GETRECTANGULARSELECTIONCARET = 2589,
+
+        //!
+        SCI_SETRECTANGULARSELECTIONANCHOR = 2590,
+
+        //!
+        SCI_GETRECTANGULARSELECTIONANCHOR = 2591,
+
+        //!
+        SCI_SETRECTANGULARSELECTIONCARETVIRTUALSPACE = 2592,
+
+        //!
+        SCI_GETRECTANGULARSELECTIONCARETVIRTUALSPACE = 2593,
+
+        //!
+        SCI_SETRECTANGULARSELECTIONANCHORVIRTUALSPACE = 2594,
+
+        //!
+        SCI_GETRECTANGULARSELECTIONANCHORVIRTUALSPACE = 2595,
+
+        //!
+        SCI_SETVIRTUALSPACEOPTIONS = 2596,
+
+        //!
+        SCI_GETVIRTUALSPACEOPTIONS = 2597,
+
+        //!
+        SCI_SETRECTANGULARSELECTIONMODIFIER = 2598,
+
+        //!
+        SCI_GETRECTANGULARSELECTIONMODIFIER = 2599,
+
+        //!
+        SCI_SETADDITIONALSELFORE = 2600,
+
+        //!
+        SCI_SETADDITIONALSELBACK = 2601,
+
+        //!
+        SCI_SETADDITIONALSELALPHA = 2602,
+
+        //!
+        SCI_GETADDITIONALSELALPHA = 2603,
+
+        //!
+        SCI_SETADDITIONALCARETFORE = 2604,
+
+        //!
+        SCI_GETADDITIONALCARETFORE = 2605,
+
+        //!
+        SCI_ROTATESELECTION = 2606,
+
+        //!
+        SCI_SWAPMAINANCHORCARET = 2607,
+
+        //!
+        SCI_SETFONTQUALITY = 2611,
+
+        //!
+        SCI_GETFONTQUALITY = 2612,
+
+        //!
+        SCI_SETFIRSTVISIBLELINE = 2613,
+
+        //!
+        SCI_SETMULTIPASTE = 2614,
+
+        //!
+        SCI_GETMULTIPASTE = 2615,
+
+        //!
+        SCI_GETTAG = 2616,
+
+        //!
+        SCI_CHANGELEXERSTATE = 2617,
+
+        //!
+        SCI_CONTRACTEDFOLDNEXT = 2618,
+
+        //!
+        SCI_VERTICALCENTRECARET = 2619,
+
+        //!
+        SCI_MOVESELECTEDLINESUP = 2620,
+
+        //!
+        SCI_MOVESELECTEDLINESDOWN = 2621,
+
+        //!
+        SCI_SETIDENTIFIER = 2622,
+
+        //!
+        SCI_GETIDENTIFIER = 2623,
+
+        //! This message sets the width of an RGBA image specified by a future
+        //! call to SCI_MARKERDEFINERGBAIMAGE or SCI_REGISTERRGBAIMAGE.
+        //!
+        //! \sa SCI_RGBAIMAGESETHEIGHT, SCI_MARKERDEFINERGBAIMAGE,
+        //! SCI_REGISTERRGBAIMAGE.
+        SCI_RGBAIMAGESETWIDTH = 2624,
+
+        //! This message sets the height of an RGBA image specified by a future
+        //! call to SCI_MARKERDEFINERGBAIMAGE or SCI_REGISTERRGBAIMAGE.
+        //!
+        //! \sa SCI_RGBAIMAGESETWIDTH, SCI_MARKERDEFINERGBAIMAGE,
+        //! SCI_REGISTERRGBAIMAGE.
+        SCI_RGBAIMAGESETHEIGHT = 2625,
+
+        //! This message sets the symbol used to draw one of the 32 markers to
+        //! an RGBA image.  RGBA images use the SC_MARK_RGBAIMAGE marker
+        //! symbol.
+        //! \a wParam is the number of the marker.
+        //! \a lParam is a pointer to a QImage instance.  Note that in other
+        //! ports of Scintilla this is a pointer to raw RGBA image data.
+        //!
+        //! \sa SCI_MARKERDEFINE, SCI_MARKERDEFINEPIXMAP
+        SCI_MARKERDEFINERGBAIMAGE = 2626,
+
+        //! This message takes a copy of an image and registers it so that it
+        //! can be refered to by a unique integer identifier.
+        //! \a wParam is the image's identifier.
+        //! \a lParam is a pointer to a QImage instance.  Note that in other
+        //! ports of Scintilla this is a pointer to raw RGBA image data.
+        //!
+        //! \sa SCI_CLEARREGISTEREDIMAGES, SCI_REGISTERIMAGE
+        SCI_REGISTERRGBAIMAGE = 2627,
+
+        //!
+        SCI_SCROLLTOSTART = 2628,
+
+        //!
+        SCI_SCROLLTOEND = 2629,
+
         //!
         SCI_STARTRECORD = 3001,
 
@@ -1658,7 +2033,25 @@ public:
         SCI_GETPROPERTYINT = 4010,
 
         //!
-        SCI_GETSTYLEBITSNEEDED = 4011
+        SCI_GETSTYLEBITSNEEDED = 4011,
+
+        //!
+        SCI_GETLEXERLANGUAGE = 4012,
+
+        //!
+        SCI_PRIVATELEXERCALL = 4013,
+
+        //!
+        SCI_PROPERTYNAMES = 4014,
+
+        //!
+        SCI_PROPERTYTYPE = 4015,
+
+        //!
+        SCI_DESCRIBEPROPERTY = 4016,
+
+        //!
+        SCI_DESCRIBEKEYWORDSETS = 4017
     };
 
     enum
@@ -1668,6 +2061,49 @@ public:
         SC_ALPHA_NOALPHA = 256
     };
 
+    enum
+    {
+        SC_CARETSTICKY_OFF = 0,
+        SC_CARETSTICKY_ON = 1,
+        SC_CARETSTICKY_WHITESPACE = 2
+    };
+
+    enum
+    {
+        SC_EFF_QUALITY_MASK = 0x0f,
+        SC_EFF_QUALITY_DEFAULT = 0,
+        SC_EFF_QUALITY_NON_ANTIALIASED = 1,
+        SC_EFF_QUALITY_ANTIALIASED = 2,
+        SC_EFF_QUALITY_LCD_OPTIMIZED = 3
+    };
+
+    enum
+    {
+        SC_MARGINOPTION_NONE = 0x00,
+        SC_MARGINOPTION_SUBLINESELECT = 0x01
+    };
+
+    enum
+    {
+        SC_MULTIPASTE_ONCE = 0,
+        SC_MULTIPASTE_EACH = 1
+    };
+
+    enum
+    {
+        SC_TYPE_BOOLEAN = 0,
+        SC_TYPE_INTEGER = 1,
+        SC_TYPE_STRING = 2
+    };
+
+    enum
+    {
+        SC_UPDATE_CONTENT = 0x01,
+        SC_UPDATE_SELECTION = 0x02,
+        SC_UPDATE_V_SCROLL = 0x04,
+        SC_UPDATE_H_SCROLL = 0x08
+    };
+
     enum
     {
         SC_WRAPVISUALFLAG_NONE = 0x0000,
@@ -1689,7 +2125,15 @@ public:
     {
         SC_SEL_STREAM = 0,
         SC_SEL_RECTANGLE = 1,
-        SC_SEL_LINES = 2
+        SC_SEL_LINES = 2,
+        SC_SEL_THIN = 3
+    };
+
+    enum
+    {
+        SCVS_NONE = 0,
+        SCVS_RECTANGULARSELECTION = 1,
+        SCVS_USERACCESSIBLE = 2
     };
 
     enum
@@ -1787,8 +2231,8 @@ public:
         //! A drawn minus sign in a connected circle.
         SC_MARK_CIRCLEMINUSCONNECTED = 21,
 
-        //! No symbol is drawn but the line of text is drawn with the
-        //! same background colour.
+        //! No symbol is drawn but the line is drawn with the same background
+        //! color as the marker's.
         SC_MARK_BACKGROUND = 22,
 
         //! Three drawn dots.
@@ -1800,9 +2244,23 @@ public:
         //! An XPM format pixmap.
         SC_MARK_PIXMAP = 25,
 
-        //! A full rectangle.
+        //! A full rectangle (ie. the margin background) using the marker's
+        //! background color.
         SC_MARK_FULLRECT = 26,
 
+        //! A left rectangle (ie. the left part of the margin background) using
+        //! the marker's background color.
+        SC_MARK_LEFTRECT = 27,
+
+        //! The value is available for plugins to use.
+        SC_MARK_AVAILABLE = 28,
+
+        //! The line is underlined using the marker's background color.
+        SC_MARK_UNDERLINE = 29,
+
+        //! A RGBA format image.
+        SC_MARK_RGBAIMAGE = 30,
+
         //! Characters can be used as symbols by adding this to the ASCII value
         //! of the character.
         SC_MARK_CHARACTER = 10000
@@ -1838,7 +2296,13 @@ public:
 
         //! The margin's background color will be set to the default foreground
         //! color.
-        SC_MARGIN_FORE = 3
+        SC_MARGIN_FORE = 3,
+
+        //! The margin will display text.
+        SC_MARGIN_TEXT = 4,
+
+        //! The margin will display right justified text.
+        SC_MARGIN_RTEXT = 5
     };
 
     enum
@@ -1911,7 +2375,6 @@ public:
 
     enum
     {
-        INDIC_MAX = 31,
         INDIC_PLAIN = 0,
         INDIC_SQUIGGLE = 1,
         INDIC_TT = 2,
@@ -1920,7 +2383,14 @@ public:
         INDIC_HIDDEN = 5,
         INDIC_BOX = 6,
         INDIC_ROUNDBOX = 7,
+        INDIC_STRAIGHTBOX = 8,
+        INDIC_DASH = 9,
+        INDIC_DOTS = 10,
+        INDIC_SQUIGGLELOW = 11,
+        INDIC_DOTBOX = 12,
+
         INDIC_CONTAINER = 8,
+        INDIC_MAX = 31,
         INDIC0_MASK = 0x20,
         INDIC1_MASK = 0x40,
         INDIC2_MASK = 0x80,
@@ -1950,16 +2420,11 @@ public:
         SC_FOLDLEVELBASE = 0x00400,
         SC_FOLDLEVELWHITEFLAG = 0x01000,
         SC_FOLDLEVELHEADERFLAG = 0x02000,
-        SC_FOLDLEVELBOXHEADERFLAG = 0x04000,
-        SC_FOLDLEVELBOXFOOTERFLAG = 0x08000,
-        SC_FOLDLEVELCONTRACTED = 0x10000,
-        SC_FOLDLEVELUNINDENT = 0x20000,
         SC_FOLDLEVELNUMBERMASK = 0x00fff
     };
 
     enum
     {
-        SC_FOLDFLAG_BOX = 0x0001,
         SC_FOLDFLAG_LINEBEFORE_EXPANDED = 0x0002,
         SC_FOLDFLAG_LINEBEFORE_CONTRACTED = 0x0004,
         SC_FOLDFLAG_LINEAFTER_EXPANDED = 0x0008,
@@ -1979,6 +2444,13 @@ public:
         SC_WRAP_CHAR = 2
     };
 
+    enum
+    {
+        SC_WRAPINDENT_FIXED = 0,
+        SC_WRAPINDENT_SAME = 1,
+        SC_WRAPINDENT_INDENT = 2
+    };
+
     enum
     {
         SC_CACHE_NONE = 0,
@@ -1987,6 +2459,13 @@ public:
         SC_CACHE_DOCUMENT = 3
     };
 
+    enum
+    {
+        ANNOTATION_HIDDEN = 0,
+        ANNOTATION_STANDARD = 1,
+        ANNOTATION_BOXED = 2
+    };
+
     enum
     {
         EDGE_NONE = 0,
@@ -1997,7 +2476,14 @@ public:
     enum
     {
         SC_CURSORNORMAL = -1,
-        SC_CURSORWAIT = 4
+        SC_CURSORARROW = 2,
+        SC_CURSORWAIT = 4,
+        SC_CURSORREVERSEARROW = 7
+    };
+
+    enum
+    {
+        UNDO_MAY_COALESCE = 1
     };
 
     enum
@@ -2039,7 +2525,11 @@ public:
         SC_STARTACTION = 0x2000,
         SC_MOD_CHANGEINDICATOR = 0x4000,
         SC_MOD_CHANGELINESTATE = 0x8000,
-        SC_MODEVENTMASKALL = 0xffff
+        SC_MOD_CHANGEMARGIN = 0x10000,
+        SC_MOD_CHANGEANNOTATION = 0x20000,
+        SC_MOD_CONTAINER = 0x40000,
+        SC_MOD_LEXERSTATE = 0x80000,
+        SC_MODEVENTMASKALL = 0xfffff
     };
 
     enum
@@ -2075,11 +2565,19 @@ public:
         //! Shift key.
         SCMOD_SHIFT = 1,
 
-        //! Control key.
+        //! Control key (the Command key on OS/X, the Ctrl key on other
+        //! platforms).
         SCMOD_CTRL = 2,
 
         //! Alt key.
-        SCMOD_ALT = 4
+        SCMOD_ALT = 4,
+
+        //! This is the same as SCMOD_META on all platforms.
+        SCMOD_SUPER = 8,
+
+        //! Meta key (the Ctrl key on OS/X, the Windows key on other
+        //! platforms).
+        SCMOD_META = 16
     };
 
     //! This enum defines the different language lexers.
@@ -2345,7 +2843,52 @@ public:
         SCLEX_ASYMPTOTE = 85,
 
         //! Select the R lexer.
-        SCLEX_R = 86
+        SCLEX_R = 86,
+
+        //! Select the MagikSF lexer.
+        SCLEX_MAGIK = 87,
+
+        //! Select the PowerShell lexer.
+        SCLEX_POWERSHELL = 88,
+
+        //! Select the MySQL lexer.
+        SCLEX_MYSQL = 89,
+
+        //! Select the gettext .po file lexer.
+        SCLEX_PO = 90,
+
+        //! Select the TAL lexer.
+        SCLEX_TAL = 91,
+
+        //! Select the COBOL lexer.
+        SCLEX_COBOL = 92,
+
+        //! Select the TACL lexer.
+        SCLEX_TACL = 93,
+
+        //! Select the Sorcus lexer.
+        SCLEX_SORCUS = 94,
+
+        //! Select the PowerPro lexer.
+        SCLEX_POWERPRO = 95,
+
+        //! Select the Nimrod lexer.
+        SCLEX_NIMROD = 96,
+
+        //! Select the SML lexer.
+        SCLEX_SML = 97,
+
+        //! Select the Markdown lexer.
+        SCLEX_MARKDOWN = 98,
+
+        //! Select the txt2tags lexer.
+        SCLEX_TXT2TAGS = 99,
+
+        //! Select the 68000 assembler lexer.
+        SCLEX_A68K = 100,
+
+        //! Select the Modula 3 lexer.
+        SCLEX_MODULA = 101
     };
 
     //! Construct an empty QsciScintillaBase with parent \a parent.
@@ -2404,6 +2947,10 @@ public:
     long SendScintilla(unsigned int msg, unsigned long wParam,
             const QPixmap &lParam) const;
 
+    //! \overload
+    long SendScintilla(unsigned int msg, unsigned long wParam,
+            const QImage &lParam) const;
+
     //! Send the Scintilla message \a msg and return a pointer result.
     void *SendScintillaPtrResult(unsigned int msg) const;
 
@@ -2414,12 +2961,23 @@ signals:
     //! deselected.
     void QSCN_SELCHANGED(bool yes);
 
+    //! This signal is emitted when the user cancels an auto-completion list.
+    //!
+    //! \sa SCN_AUTOCSELECTION()
+    void SCN_AUTOCCANCELLED();
+
+    //! This signal is emitted when the user deletes a character when an
+    //! auto-completion list is active.
+    void SCN_AUTOCCHARDELETED();
+
     //! This signal is emitted when the user selects an item in an
     //! auto-completion list.  It is emitted before the selection is inserted.
     //! The insertion can be cancelled by sending an SCI_AUTOCANCEL message
     //! from a connected slot.
     //! \a selection is the text of the selection.
     //! \a position is the start position of the word being completed.
+    //!
+    //! \sa SCN_AUTOCCANCELLED()
     void SCN_AUTOCSELECTION(const char *selection, int position);
 
     //! This signal is emitted when the document has changed for any reason.
@@ -2463,6 +3021,13 @@ signals:
     //! when the user double clicked.
     void SCN_HOTSPOTDOUBLECLICK(int position, int modifiers);
 
+    //! This signal is emitted when the user releases the mouse button on text
+    //! in a style with the hotspot attribute set.
+    //! \a position is the position in the text where the release occured.
+    //! \a modifiers is the logical or of the modifier keys that were pressed
+    //! when the user released the button.
+    void SCN_HOTSPOTRELEASECLICK(int position, int modifiers);
+
     //! This signal is emitted when the user clicks on text that has an
     //! indicator.
     //! \a position is the position in the text where the click occured.
@@ -2492,7 +3057,7 @@ signals:
     void SCN_MARGINCLICK(int position, int modifiers, int margin);
 
     //!
-    void SCN_MODIFIED(int, int, const char *, int, int, int, int, int);
+    void SCN_MODIFIED(int, int, const char *, int, int, int, int, int, int, int);
 
     //! This signal is emitted when the user attempts to modify read-only
     //! text.
@@ -2523,13 +3088,16 @@ signals:
     //! This signal is emitted when a range of text needs to be syntax styled.
     //! The range is from the value returned by the SCI_GETENDSTYLED message
     //! and \a position.  It is only emitted if the currently selected lexer is
-    //! SCNLEX_CONTAINER.
+    //! SCLEX_CONTAINER.
     //!
     //! \sa SCI_COLOURISE, SCI_GETENDSTYLED
     void SCN_STYLENEEDED(int position);
 
-    //!
-    void SCN_UPDATEUI();
+    //! This signal is emitted when either the text or styling of the text has
+    //! changed or the selection range or scroll position has changed.
+    //! \a updated contains the set of SC_UPDATE_* flags describing the changes
+    //! since the signal was last emitted.
+    void SCN_UPDATEUI(int updated);
 
     //!
     void SCN_USERLISTSELECTION(const char *, int);
@@ -2538,6 +3106,32 @@ signals:
     void SCN_ZOOM();
 
 protected:
+    //! Returns true if the contents of a MIME data object can be decoded and
+    //! inserted into the document.  It is called during drag and paste
+    //! operations.
+    //! \a source is the MIME data object.
+    //!
+    //! \sa fromMimeData(), toMimeData()
+    virtual bool canInsertFromMimeData(const QMimeData *source) const;
+
+    //! Returns the text of a MIME data object.  It is called when a drag and
+    //! drop is completed and when text is pasted from the clipboard.
+    //! \a source is the MIME data object.  On return \a rectangular is set if
+    //! the text corresponds to a rectangular selection.
+    //!
+    //! \sa canInsertFromMimeData(), toMimeData()
+    virtual QByteArray fromMimeData(const QMimeData *source, bool &rectangular) const;
+
+    //! Returns a new MIME data object containing some text and whether it
+    //! corresponds to a rectangular selection.  It is called when a drag and
+    //! drop is started and when the selection is copied to the clipboard.
+    //! Ownership of the object is passed to the caller.  \a text is the text.
+    //! \a rectangular is set if the text corresponds to a rectangular
+    //! selection.
+    //!
+    //! \sa canInsertFromMimeData(), fromMimeData()
+    virtual QMimeData *toMimeData(const QByteArray &text, bool rectangular) const;
+
     //! Re-implemented to handle the context menu.
     virtual void contextMenuEvent(QContextMenuEvent *e);
 
@@ -2603,6 +3197,8 @@ private:
     QPoint triple_click_at;
     QTimer triple_click;
 
+    void acceptAction(QDropEvent *e);
+
     QsciScintillaBase(const QsciScintillaBase &);
     QsciScintillaBase &operator=(const QsciScintillaBase &);
 };
